@@ -23,6 +23,7 @@
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
  */
+#define QTC_DRAW_INTO_PIXMAPS
 
 #include <klocale.h>
 #include <QBitmap>
@@ -35,10 +36,14 @@
 #include <QPixmap>
 #include <QStyleOptionTitleBar>
 #include <QStyle>
+#ifdef QTC_DRAW_INTO_PIXMAPS
+#include <KDE/KWindowSystem>
+#endif
 #include <qdesktopwidget.h>
 #include "qtcurvehandler.h"
 #include "qtcurveclient.h"
 #include "qtcurvebutton.h"
+#include "resizecorner.h"
 #define QTC_KWIN
 #include "common.h"
 
@@ -46,11 +51,12 @@ namespace KWinQtCurve
 {
 
 QtCurveClient::QtCurveClient(KDecorationBridge *bridge, KDecorationFactory *factory)
-#if KDE_IS_VERSION(4,1,80)
+#if KDE_IS_VERSION(4,1,80) && !KDE_IS_VERSION(4,2,92)
              : KCommonDecorationUnstable(bridge, factory),
 #else
              : KCommonDecoration(bridge, factory),
 #endif
+               itsResizeGrip(0L),
                itsTitleFont(QFont())
 {
 }
@@ -85,12 +91,12 @@ int QtCurveClient::layoutMetric(LayoutMetric lm, bool respectWindowState,
         case LM_BorderBottom:
             return respectWindowState && maximized ? 0 : Handler()->borderSize();
         case LM_TitleEdgeTop:
-            return respectWindowState && maximized ? 0 : 3;
+            return respectWindowState && maximized ? 0 : Handler()->borderEdgeSize();
         case LM_TitleEdgeBottom:
-            return /*respectWindowState && maximized ? 1 : */ 3;
+            return /*respectWindowState && maximized ? 1 : */ Handler()->borderEdgeSize();
         case LM_TitleEdgeLeft:
         case LM_TitleEdgeRight:
-            return respectWindowState && maximized ? 0 : 3;
+            return respectWindowState && maximized ? 0 : Handler()->borderEdgeSize();
         case LM_TitleBorderLeft:
         case LM_TitleBorderRight:
             return 5;
@@ -131,14 +137,38 @@ void QtCurveClient::init()
     itsTitleFont = isToolWindow() ? Handler()->titleFontTool() : Handler()->titleFont();
 
     KCommonDecoration::init();
+    // If WA_PaintOnScreen is set to false when not compositnig then get redraw errors
+    // ...if set to true when compositing, then sometimes only part of the titlebar is updated!
+    // ...hence in reset() we need to alter this setting
+    // ... :-(
+    widget()->setAttribute(Qt::WA_PaintOnScreen, !KWindowSystem::compositingActive());
     widget()->setAutoFillBackground(false);
-    widget()->setAttribute(Qt::WA_OpaquePaintEvent);
-    widget()->setAttribute(Qt::WA_PaintOnScreen, false);
+    widget()->setAttribute(Qt::WA_OpaquePaintEvent, true);
+    
+    if(Handler()->showResizeGrip() && isResizable())
+        itsResizeGrip=new ResizeCorner(this, KDecoration::options()->color(KDecoration::ColorTitleBar, isActive()));
+}
+
+void QtCurveClient::maximizeChange()
+{
+    reset(SettingBorder);
+    KCommonDecoration::maximizeChange();
+}
+
+void QtCurveClient::activeChange()
+{
+    if (itsResizeGrip)
+    {
+        itsResizeGrip->setColor(KDecoration::options()->color(KDecoration::ColorTitleBar, isActive()));
+        itsResizeGrip->update();
+    }
+    KCommonDecoration::activeChange();
 }
 
 void QtCurveClient::drawBtnBgnd(QPainter *p, const QRect &r, bool active)
 {
-    int    state(active ? 1 : 0);
+    bool   mximised(maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows());
+    int    state((active ? 1 : 0)+((mximised ? 1 : 0)<<1));
     QColor col(KDecoration::options()->color(KDecoration::ColorTitleBar, active));
     bool   diffSize(itsButtonBackground[state].pix.width()!=r.width() ||
                     itsButtonBackground[state].pix.height()!=r.height());
@@ -152,8 +182,9 @@ void QtCurveClient::drawBtnBgnd(QPainter *p, const QRect &r, bool active)
         QRect                br(r);
         QStyleOptionTitleBar opt;
         QPainter             pixPainter(&(itsButtonBackground[state].pix));
+        int                  border(Handler()->borderEdgeSize());
 
-        br.adjust(-3, -3, 3, 3);
+        br.adjust(-6, -border, 6, 0);
         opt.rect=br;
 
         opt.state=QStyle::State_Horizontal|QStyle::State_Enabled|QStyle::State_Raised|
@@ -170,13 +201,19 @@ void QtCurveClient::drawBtnBgnd(QPainter *p, const QRect &r, bool active)
 
 void QtCurveClient::paintEvent(QPaintEvent *e)
 {
+#ifdef QTC_DRAW_INTO_PIXMAPS
+    bool                 compositing=KWindowSystem::compositingActive();
+#endif
     QPainter             painter(widget());
-    QRect                r(widget()->rect());
+    QRect                r(widget()->rect()),
+                         rx(r);
     QStyleOptionTitleBar opt;
     bool                 active(isActive()),
                          colorTitleOnly(Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarColorTopOnly,
                                         NULL, NULL)),
-                         mximised(maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows());
+                         mximised(maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows()),
+                         roundBottom(Handler()->roundBottom()),
+                         noBorder(Handler()->noBorder());
     const int            maximiseOffset(mximised ? 3 : 0),
                          titleHeight(layoutMetric(LM_TitleHeight)),
                          titleEdgeTop(layoutMetric(LM_TitleEdgeTop)),
@@ -186,14 +223,15 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
                          titleBarHeight(titleHeight+titleEdgeTop+titleEdgeBottom+maximiseOffset),
                          borderSize(Handler()->borderSize()),
                          round=Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_Round, NULL, NULL);
-    int                  rectX, rectY, rectX2, rectY2;
+    int                  rectX, rectY, rectX2, rectY2,
+                         border(Handler()->borderEdgeSize());
 
     r.getCoords(&rectX, &rectY, &rectX2, &rectY2);
 
-    QColor    col(KDecoration::options()->color(KDecoration::ColorTitleBar, active)),
-              windowCol(widget()->palette().color(QPalette::Window));
+    QColor col(KDecoration::options()->color(KDecoration::ColorTitleBar, active)),
+           windowCol(widget()->palette().color(QPalette::Window));
 
-#if KDE_IS_VERSION(4,1,80)
+#if KDE_IS_VERSION(4,1,80) && !KDE_IS_VERSION(4,2,80)
     if(!(Handler()->coloredShadow() && shadowsActive() && active))
 #endif
     {
@@ -203,34 +241,17 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
     painter.setClipRegion(e->region().intersected(getMask(round, r.width(), r.height())));
     painter.fillRect(r, colorTitleOnly ? windowCol : col);
 
-    if(ROUND_FULL==round)
-    {
-        QColor cornerCol(colorTitleOnly ? windowCol : col);
-        painter.setPen(windowCol);
-        painter.drawRect(r.x()+borderSize-1, r.y()+borderSize-1,
-                         r.x()+r.width()-((borderSize*2)-1), r.y()+r.height()-((borderSize*2)-1));
-        painter.setPen(cornerCol);
-        painter.drawPoint(r.x()+borderSize-1, r.y()+r.height()-(borderSize));
-        painter.drawPoint(r.x()+r.width()-borderSize, r.y()+r.height()-borderSize);
-        cornerCol.setAlphaF(0.5);
-        painter.setPen(cornerCol);
-        painter.drawPoint(r.x()+borderSize, r.y()+r.height()-(borderSize));
-        painter.drawPoint(r.x()+borderSize-1, r.y()+r.height()-(borderSize+1));
-        painter.drawPoint(r.x()+r.width()-borderSize-1, r.y()+r.height()-borderSize);
-        painter.drawPoint(r.x()+r.width()-borderSize, r.y()+r.height()-(borderSize+1));
-    }
-
     opt.init(widget());
 
     if(mximised)
-        r.adjust(-3, -3, 3, 0);
+        r.adjust(-3, -border, 3, 0);
     opt.palette.setColor(QPalette::Button, col);
     opt.palette.setColor(QPalette::Window, windowCol);
     opt.rect=QRect(r.x(), r.y()+6, r.width(), r.height()-6);
     opt.state=QStyle::State_Horizontal|QStyle::State_Enabled|QStyle::State_Raised|
              (active ? QStyle::State_Active : QStyle::State_None)|QtC_StateKWin;
 
-#if KDE_IS_VERSION(4,1,80)
+#if KDE_IS_VERSION(4,1,80) && !KDE_IS_VERSION(4,2,80)
     if(Handler()->coloredShadow() && shadowsActive())
     {
         opt.state|=QtC_StateKWinShadows;
@@ -238,7 +259,50 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
             opt.state|=QtC_StateKWinHighlight;
     }
 #endif
-    Handler()->wStyle()->drawPrimitive(QStyle::PE_FrameWindow, &opt, &painter, widget());
+
+    if(!roundBottom)
+        opt.state|=QtCStateKWinNotFull;
+    if(noBorder)
+        opt.state|=QtCStateKWinNoBorder;
+    else
+    {
+#ifdef QTC_DRAW_INTO_PIXMAPS
+        if(!compositing)
+        {
+            // For some reason, on Jaunty drawing directly is *hideously* slow on intel graphics card!
+            QPixmap pix(32, 32);
+            QPainter p2(&pix);
+            opt.rect=QRect(0, 0, pix.width(), pix.height());
+            p2.fillRect(opt.rect, colorTitleOnly ? windowCol : col);
+            Handler()->wStyle()->drawPrimitive(QStyle::PE_FrameWindow, &opt, &p2, widget());
+            p2.end();
+            painter.drawTiledPixmap(r.x(), r.y()+10, 2, r.height()-18, pix.copy(0, 8, 2, 16));
+            painter.drawTiledPixmap(r.x()+r.width()-2, r.y()+8, 2, r.height()-16, pix.copy(pix.width()-2, 8, 2, 16));
+            painter.drawTiledPixmap(r.x()+8, r.y()+r.height()-2, r.width()-16, 2, pix.copy(8, pix.height()-2, 16, 2));
+            painter.drawPixmap(r.x(), r.y()+r.height()-8, pix.copy(0, 24, 8, 8));
+            painter.drawPixmap(r.x()+r.width()-8, r.y()+r.height()-8, pix.copy(24, 24, 8, 8));
+        }
+        else
+#endif
+            Handler()->wStyle()->drawPrimitive(QStyle::PE_FrameWindow, &opt, &painter, widget());
+    }
+
+    if(round>=ROUND_FULL && !colorTitleOnly && col!=windowCol && roundBottom)
+    {
+        QColor cornerCol(col);
+        painter.setPen(windowCol);
+        painter.drawRect(rx.x()+borderSize-1, rx.y()+borderSize-1,
+                         rx.x()+rx.width()-((borderSize*2)-1), rx.y()+rx.height()-((borderSize*2)-1));
+        painter.setPen(cornerCol);
+        painter.drawPoint(rx.x()+borderSize-1, rx.y()+rx.height()-(borderSize));
+        painter.drawPoint(rx.x()+rx.width()-borderSize, rx.y()+rx.height()-borderSize);
+        cornerCol.setAlphaF(0.5);
+        painter.setPen(cornerCol);
+        painter.drawPoint(rx.x()+borderSize, rx.y()+rx.height()-(borderSize));
+        painter.drawPoint(rx.x()+borderSize-1, rx.y()+rx.height()-(borderSize+1));
+        painter.drawPoint(rx.x()+rx.width()-borderSize-1, rx.y()+rx.height()-borderSize);
+        painter.drawPoint(rx.x()+rx.width()-borderSize, rx.y()+rx.height()-(borderSize+1));
+    }
 
     opt.palette.setColor(QPalette::Button, col);
     opt.rect=QRect(r.x(), r.y(), r.width(), titleBarHeight);
@@ -246,32 +310,130 @@ void QtCurveClient::paintEvent(QPaintEvent *e)
     if(KDecoration::options()->color(KDecoration::ColorTitleBar, true)!=windowCol ||
        KDecoration::options()->color(KDecoration::ColorTitleBar, false)!=windowCol)
        opt.titleBarState|=QtCStateKWinDrawLine;
-    Handler()->wStyle()->drawComplexControl(QStyle::CC_TitleBar, &opt, &painter, widget());
+#ifdef QTC_DRAW_INTO_PIXMAPS
+    if(!compositing)
+    {
+        QPixmap  tPix(32, titleBarHeight);
+        QPainter tPainter(&tPix);
+        opt.rect=QRect(0, 0, tPix.width(), tPix.height());
+        Handler()->wStyle()->drawComplexControl(QStyle::CC_TitleBar, &opt, &tPainter, widget());
+        tPainter.end();
+        painter.drawTiledPixmap(r.x()+12, r.y(), r.width()-24, tPix.height(), tPix.copy(8, 0, 16, tPix.height()));
+        painter.drawPixmap(r.x(), r.y(), tPix.copy(0, 0, 16, tPix.height()));
+        painter.drawPixmap(r.x()+r.width()-16, r.y(), tPix.copy(tPix.width()-16, 0, 16, tPix.height()));
+    }
+    else
+#endif
+        Handler()->wStyle()->drawComplexControl(QStyle::CC_TitleBar, &opt, &painter, widget());
 
     itsCaptionRect = captionRect(); // also update itsCaptionRect!
+    bool     showIcon=TITLEBAR_ICON_NEXT_TO_TITLE==Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleBarIcon,
+                                                                                    0L, 0L);
+    int     iconSize=showIcon ? Handler()->wStyle()->pixelMetric(QStyle::PM_SmallIconSize) : 0,
+            iconX=itsCaptionRect.x();
+    QPixmap menuIcon;
 
+    if(showIcon)
+    {
+        menuIcon=icon().pixmap(iconSize);
+
+        if(menuIcon.isNull())
+            showIcon=false;
+    }
+        
     if(!caption().isEmpty())
     {
+        static const int constPad=4;
+
         painter.setFont(itsTitleFont);
 
-        QString       str(painter.fontMetrics().elidedText(caption(), Qt::ElideRight,
-                                                           itsCaptionRect.width(), QPalette::WindowText));
-        Qt::Alignment hAlign((Qt::Alignment)Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleAlignment,
-                                                                                 0L, 0L)),
+        QFontMetrics  fm(painter.fontMetrics());
+        QString       str(fm.elidedText(caption(), Qt::ElideRight,
+                            itsCaptionRect.width()-(showIcon ? iconSize+constPad : 0), QPalette::WindowText));
+        Qt::Alignment hAlign((Qt::Alignment)Handler()->wStyle()->pixelMetric((QStyle::PixelMetric)QtC_TitleAlignment, 0L, 0L)),
                       alignment(Qt::AlignVCenter|hAlign);
         const int     titleEdgeBottomBottom(rectY+titleEdgeTop+titleHeight+titleEdgeBottom);
-        QRect         textRect(Qt::AlignHCenter==hAlign
-                                    ? QRect(rectX+titleEdgeLeft, rectY+titleEdgeTop,
+        bool          alignFull(Qt::AlignHCenter==hAlign),
+                      reverse=Qt::RightToLeft==QApplication::layoutDirection(),
+                      iconRight((!reverse && alignment&Qt::AlignRight) || (reverse && alignment&Qt::AlignLeft));
+        QRect         textRect(alignFull
+                                    ? QRect(rectX+titleEdgeLeft, itsCaptionRect.y(),
                                             rectX2-titleEdgeRight-(rectX+titleEdgeLeft),
-                                            titleEdgeBottomBottom-(rectY+titleEdgeTop))
+                                            itsCaptionRect.height())
                                     : itsCaptionRect);
+        int           textWidth=alignFull || (showIcon && alignment&Qt::AlignHCenter)
+                                    ? fm.boundingRect(str).width()+(showIcon ? iconSize+constPad : 0) : 0;
 
-        painter.setClipRect(itsCaptionRect);
-        painter.setPen(shadowColor(KDecoration::options()->color(KDecoration::ColorFont, active)));
+        if(alignFull)
+            if(itsCaptionRect.left()>((textRect.width()-textWidth)>>1))
+            {
+                alignment=Qt::AlignVCenter|Qt::AlignLeft;
+                textRect=itsCaptionRect;
+            }
+            else if(itsCaptionRect.right()<((textRect.width()+textWidth)>>1))
+            {
+                alignment=Qt::AlignVCenter|Qt::AlignRight;
+                textRect=itsCaptionRect;
+            }
+
+        if(showIcon)
+            if(alignment&Qt::AlignHCenter)
+            {
+                if(reverse)
+                {
+                    iconX=((textRect.width()-textWidth)/2.0)+0.5+textWidth+iconSize;
+                    textRect.setX(textRect.x()-(iconSize+constPad));
+//                     iconX=((textRect.width()-textWidth)/2.0)+0.5+(textWidth-constPad);
+//                     textRect.setX(iconX-(textWidth-iconSize));
+//                     textRect.setWidth(textWidth-iconSize);
+//                     alignment=Qt::AlignVCenter|Qt::AlignRight;
+                }
+                else
+                {
+                    iconX=((textRect.width()-textWidth)/2.0)+0.5;
+                    textRect.setX(iconX+iconSize+constPad);
+                    alignment=Qt::AlignVCenter|Qt::AlignLeft;
+                }
+            }
+            else if((!reverse && alignment&Qt::AlignLeft) || (reverse && alignment&Qt::AlignRight))
+            {
+                iconX=textRect.x();
+                textRect.setX(textRect.x()+(iconSize+constPad));
+            }
+            else if((!reverse && alignment&Qt::AlignRight) || (reverse && alignment&Qt::AlignLeft))
+            {
+                if(iconRight)
+                {
+                    iconX=textRect.x()+textRect.width()-iconSize;
+                    textRect.setWidth(textRect.width()-(iconSize+constPad));
+                }
+                else
+                {
+                    iconX=textRect.x()+textRect.width()-textWidth;
+                    if(iconX<textRect.x())
+                        iconX=textRect.x();
+                }
+            }
+
+        painter.setClipRect(itsCaptionRect.adjusted(-2, 0, 2, 0));
+        QColor shadow(Qt::black);
+        shadow.setAlphaF(WINDOW_TEXT_SHADOW_ALPHA);
+        painter.setPen(shadow);
         painter.drawText(textRect.adjusted(1, 1, 1, 1), alignment, str);
-        painter.setPen(KDecoration::options()->color(KDecoration::ColorFont, active));
+
+        QColor color(KDecoration::options()->color(KDecoration::ColorFont, active));
+
+        if (!active && QTC_DARK_WINDOW_TEXT(color))
+             color.setAlpha((color.alpha() * 180) >> 8);
+
+        painter.setPen(color);
         painter.drawText(textRect, alignment, str);
+        painter.setClipping(false);
     }
+
+    if(showIcon && iconX>=0)
+        painter.drawPixmap(iconX, itsCaptionRect.y()+((itsCaptionRect.height()-iconSize)/2)+1, menuIcon);
+
     painter.end();
 }
 
@@ -299,16 +461,30 @@ QRegion QtCurveClient::getMask(int round, int w, int h, bool maximised) const
         default: // ROUND_FULL
         {
             QRegion mask(5, 0, w-10, h);
+            bool    roundBottom=Handler()->roundBottom();
 
-            mask += QRegion(0, 5, 1, h-10);
-            mask += QRegion(1, 3, 1, h-6);
-            mask += QRegion(2, 2, 1, h-4);
-            mask += QRegion(3, 1, 2, h-2);
-
-            mask += QRegion(w-1, 5, 1, h-10);
-            mask += QRegion(w-2, 3, 1, h-6);
-            mask += QRegion(w-3, 2, 1, h-4);
-            mask += QRegion(w-5, 1, 2, h-2);
+            if(roundBottom)
+            {
+                mask += QRegion(0, 5, 1, h-10);
+                mask += QRegion(1, 3, 1, h-6);
+                mask += QRegion(2, 2, 1, h-4);
+                mask += QRegion(3, 1, 2, h-2);
+                mask += QRegion(w-1, 5, 1, h-10);
+                mask += QRegion(w-2, 3, 1, h-6);
+                mask += QRegion(w-3, 2, 1, h-4);
+                mask += QRegion(w-5, 1, 2, h-2);
+            }
+            else
+            {
+                mask += QRegion(0, 5, 1, h-5);
+                mask += QRegion(1, 3, 1, h-2);
+                mask += QRegion(2, 2, 1, h-1);
+                mask += QRegion(3, 1, 2, h);
+                mask += QRegion(w-1, 5, 1, h-5);
+                mask += QRegion(w-2, 3, 1, h-2);
+                mask += QRegion(w-3, 2, 1, h-1);
+                mask += QRegion(w-5, 1, 2, h);
+            }
 
             return mask;
         }
@@ -354,7 +530,7 @@ bool QtCurveClient::eventFilter(QObject *o, QEvent *e)
     return KCommonDecoration::eventFilter(o, e);
 }
 
-#if KDE_IS_VERSION(4,1,80)
+#if KDE_IS_VERSION(4,1,80) && !KDE_IS_VERSION(4,2,92)
 // Taken form Oxygen! rev873805
 QList<QRect> QtCurveClient::shadowQuads(ShadowType type) const
 {
@@ -391,13 +567,26 @@ double QtCurveClient::shadowOpacity(ShadowType type) const
     
 void QtCurveClient::reset(unsigned long changed)
 {
-    if (changed & SettingColors)
+    // Set note in init() above
+    if(0==changed)
+        widget()->setAttribute(Qt::WA_PaintOnScreen, !KWindowSystem::compositingActive());
+    
+    if (changed&SettingBorder)
+        if (maximizeMode() == MaximizeFull)
+        {
+            if (!options()->moveResizeMaximizedWindows() && itsResizeGrip)
+                itsResizeGrip->hide();
+        }
+        else if (itsResizeGrip)
+                itsResizeGrip->show();
+
+    if (changed&SettingColors)
     {
         // repaint the whole thing
         widget()->update();
         updateButtons();
     }
-    else if (changed & SettingFont)
+    else if (changed&SettingFont)
     {
         // font has changed -- update title height and font
         itsTitleFont = isToolWindow() ? Handler()->titleFontTool() : Handler()->titleFont();
@@ -405,7 +594,7 @@ void QtCurveClient::reset(unsigned long changed)
         updateLayout();
         widget()->update();
     }
-
+    
     KCommonDecoration::reset(changed);
 }
 
